@testset verbose=true "SphereShape" begin
    @testset verbose=true "SphereShape 2D" begin
        radius = 2.4
        particle_spacing = 0.8

        shape_names = [
            "1-particle VoxelSphere",
            "1-particle RoundSphere",
            "5-particle VoxelSphere",
            "5-particle RoundSphere",
            "7-particle RoundSphere",
            "Filled VoxelSphere",
            "Filled VoxelSphere with Cutout",
            "Hollow VoxelSphere with Multiple Layers",
            "Hollow VoxelSphere Outwards",
            "Hollow RoundSphere with One Layer",
            "Hollow RoundSphere with Multiple Layers",
            "Hollow RoundSphere Outwards",
            "Hollow RoundSphere Segment",
        ]

        shapes = [
            SphereShape(1.0, 1.0, (0.0, 0.0), 1000.0),
            SphereShape(1.0, 0.3, (0.0, 0.0), 1000.0, sphere_type=RoundSphere()),
            SphereShape(1.0, 1.5, (0.0, 0.0), 1000.0),
            SphereShape(1.0, 1.26, (0.0, 0.0), 1000.0, sphere_type=RoundSphere()),
            SphereShape(1.0, 1.5, (0.0, 0.0), 1000.0, sphere_type=RoundSphere()),
            SphereShape(particle_spacing, radius, (0.0, 0.0), 1000.0),
            SphereShape(particle_spacing, radius, (3.0, 4.0), 1000.0,
                        cutout_min=(3.0, 4.0), cutout_max=(5.0, 4.0 + 0.5particle_spacing)),
            SphereShape(particle_spacing, 4.4, (0.3, 0.4), 1000.0,
                        n_layers=2),
            SphereShape(particle_spacing, 3.6, (0.3, 0.4), 1000.0,
                        n_layers=3, layer_outwards=true),
            SphereShape(particle_spacing, radius, (-3.0, -4.0), 1000.0,
                        sphere_type=RoundSphere(), n_layers=1),
            SphereShape(particle_spacing, radius, (3.0, -4), 1000.0,
                        sphere_type=RoundSphere(),
                        n_layers=2),
            SphereShape(particle_spacing, radius, (-3.0, 4.0), 1000.0,
                        sphere_type=RoundSphere(),
                        n_layers=3, layer_outwards=true),
            SphereShape(particle_spacing, radius, (3.0, -4), 1000.0,
                        sphere_type=RoundSphere(start_angle=pi, end_angle=1.5pi),
                        n_layers=3, layer_outwards=true),
        ]

        expected_coords = [
            [0.0; 0.0],
            [0.0; 0.0],
            [0.0 -1.0 0.0 1.0 0.0; -1.0 0.0 0.0 0.0 1.0],
            [0.76 0.23485292 -0.61485292 -0.61485292 0.23485292;
             0.0 0.72280295 0.44671679 -0.44671679 -0.72280295],
            # Note that this 7-particle configuration is better than the 5-particle
            # configuration of the `VoxelSphere` above. The sphere has the volume
            # of 7.07 particles, so 7 is the optimal number of particles.
            [0.0 1.0 0.5 -0.5 -1.0 -0.5 0.5;
             0.0 0.0 0.8660254 0.8660254 0.0 -0.8660254 -0.8660254],
            [-0.8 0.0 0.8 -1.6 -0.8 0.0 0.8 1.6 -1.6 -0.8 0.0 0.8 1.6 -1.6 -0.8 0.0 0.8 1.6 -0.8 0.0 0.8
             -1.6 -1.6 -1.6 -0.8 -0.8 -0.8 -0.8 -0.8 0.0 0.0 0.0 0.0 0.0 0.8 0.8 0.8 0.8 0.8 1.6 1.6 1.6],
            [2.2 3.0 3.8 1.4 2.2 3.0 3.8 4.6 1.4 2.2 1.4 2.2 3.0 3.8 4.6 2.2 3.0 3.8;
             2.4 2.4 2.4 3.2 3.2 3.2 3.2 3.2 4.0 4.0 4.8 4.8 4.8 4.8 4.8 5.6 5.6 5.6],
            [0.3 -2.1 -1.3 -0.5 0.3 1.1 1.9 2.7 -2.9 -2.1 -1.3 -0.5 1.1 1.9 2.7 3.5 -2.9 -2.1 2.7 3.5 -2.9 -2.1 2.7 3.5 -3.7 -2.9 3.5 4.3 -2.9 -2.1 2.7 3.5 -2.9 -2.1 2.7 3.5 -2.9 -2.1 -1.3 -0.5 1.1 1.9 2.7 3.5 -2.1 -1.3 -0.5 0.3 1.1 1.9 2.7 0.3;
             -3.6 -2.8 -2.8 -2.8 -2.8 -2.8 -2.8 -2.8 -2.0 -2.0 -2.0 -2.0 -2.0 -2.0 -2.0 -2.0 -1.2 -1.2 -1.2 -1.2 -0.4 -0.4 -0.4 -0.4 0.4 0.4 0.4 0.4 1.2 1.2 1.2 1.2 2.0 2.0 2.0 2.0 2.8 2.8 2.8 2.8 2.8 2.8 2.8 2.8 3.6 3.6 3.6 3.6 3.6 3.6 3.6 4.4],
            [0.3 -2.1 -1.3 -0.5 0.3 1.1 1.9 2.7 -3.7 -2.9 -2.1 -1.3 -0.5 0.3 1.1 1.9 2.7 3.5 4.3 -4.5 -3.7 -2.9 -2.1 -1.3 -0.5 1.1 1.9 2.7 3.5 4.3 5.1 -4.5 -3.7 -2.9 3.5 4.3 5.1 -5.3 -4.5 -3.7 4.3 5.1 5.9 -5.3 -4.5 -3.7 4.3 5.1 5.9 -5.3 -4.5 -3.7 4.3 5.1 5.9 -6.1 -5.3 -4.5 5.1 5.9 6.7 -5.3 -4.5 -3.7 4.3 5.1 5.9 -5.3 -4.5 -3.7 4.3 5.1 5.9 -5.3 -4.5 -3.7 4.3 5.1 5.9 -4.5 -3.7 -2.9 3.5 4.3 5.1 -4.5 -3.7 -2.9 -2.1 -1.3 -0.5 1.1 1.9 2.7 3.5 4.3 5.1 -3.7 -2.9 -2.1 -1.3 -0.5 0.3 1.1 1.9 2.7 3.5 4.3 -2.1 -1.3 -0.5 0.3 1.1 1.9 2.7 0.3;
             -6.0 -5.2 -5.2 -5.2 -5.2 -5.2 -5.2 -5.2 -4.4 -4.4 -4.4 -4.4 -4.4 -4.4 -4.4 -4.4 -4.4 -4.4 -4.4 -3.6 -3.6 -3.6 -3.6 -3.6 -3.6 -3.6 -3.6 -3.6 -3.6 -3.6 -3.6 -2.8 -2.8 -2.8 -2.8 -2.8 -2.8 -2.0 -2.0 -2.0 -2.0 -2.0 -2.0 -1.2 -1.2 -1.2 -1.2 -1.2 -1.2 -0.4 -0.4 -0.4 -0.4 -0.4 -0.4 0.4 0.4 0.4 0.4 0.4 0.4 1.2 1.2 1.2 1.2 1.2 1.2 2.0 2.0 2.0 2.0 2.0 2.0 2.8 2.8 2.8 2.8 2.8 2.8 3.6 3.6 3.6 3.6 3.6 3.6 4.4 4.4 4.4 4.4 4.4 4.4 4.4 4.4 4.4 4.4 4.4 4.4 5.2 5.2 5.2 5.2 5.2 5.2 5.2 5.2 5.2 5.2 5.2 6.0 6.0 6.0 6.0 6.0 6.0 6.0 6.8],
            [-1.0 -1.15224 -1.58579 -2.23463 -3.0 -3.76537 -4.41421 -4.84776 -5.0 -4.84776 -4.41421 -3.76537 -3.0 -2.23463 -1.58579 -1.15224;
             -4.0 -3.23463 -2.58579 -2.15224 -2.0 -2.15224 -2.58579 -3.23463 -4.0 -4.76537 -5.41421 -5.84776 -6.0 -5.84776 -5.41421 -4.76537],
            [4.2 3.91925333 3.20837781 2.4 1.87236886 1.87236886 2.4 3.20837781 3.91925333 5.0 4.84775907 4.41421356 3.76536686 3.0 2.23463314 1.58578644 1.15224093 1.0 1.15224093 1.58578644 2.23463314 3.0 3.76536686 4.41421356 4.84775907;
             -4.0 -3.22865487 -2.8182307 -2.96076952 -3.58957583 -4.41042417 -5.03923048 -5.1817693 -4.77134513 -4.0 -3.23463314 -2.58578644 -2.15224093 -2.0 -2.15224093 -2.58578644 -3.23463314 -4.0 -4.76536686 -5.41421356 -5.84775907 -6.0 -5.84775907 -5.41421356 -4.76536686],
            [-0.2 -0.31341967 -0.64449011 -1.16638994 -1.83683796 -2.60151845 -3.39848155 -4.16316204 -4.83361006 -5.35550989 -5.68658033 -5.8 -5.68658033 -5.35550989 -4.83361006 -4.16316204 -3.39848155 -2.60151845 -1.83683796 -1.16638994 -0.64449011 -0.31341967 0.6 0.50974048 0.24348792 -0.18540666 -0.75543671 -1.43801854 -2.19892464 -3.0 -3.80107536 -4.56198146 -5.24456329 -5.81459334 -6.24348792 -6.50974048 -6.6 -6.50974048 -6.24348792 -5.81459334 -5.24456329 -4.56198146 -3.80107536 -3.0 -2.19892464 -1.43801854 -0.75543671 -0.18540666 0.24348792 0.50974048 1.4 1.32929019 1.11943343 0.77717469 0.31351445 -0.25664487 -0.91497789 -1.64032522 -2.40937363 -3.19740525 -3.97909211 -4.72931014 -5.42394672 -6.04067566 -6.55967478 -6.96426302 -7.24143659 -7.38228689 -7.38228689 -7.24143659 -6.96426302 -6.55967478 -6.04067566 -5.42394672 -4.72931014 -3.97909211 -3.19740525 -2.40937363 -1.64032522 -0.91497789 -0.25664487 0.31351445 0.77717469 1.11943343 1.32929019;
             4.0 4.78885116 5.51379429 6.11609881 6.54696959 6.77150004 6.77150004 6.54696959 6.11609881 5.51379429 4.78885116 4.0 3.21114884 2.48620571 1.88390119 1.45303041 1.22849996 1.22849996 1.45303041 1.88390119 2.48620571 3.21114884 4.0 4.80107536 5.56198146 6.24456329 6.81459334 7.24348792 7.50974048 7.6 7.50974048 7.24348792 6.81459334 6.24456329 5.56198146 4.80107536 4.0 3.19892464 2.43801854 1.75543671 1.18540666 0.75651208 0.49025952 0.4 0.49025952 0.75651208 1.18540666 1.75543671 2.43801854 3.19892464 4.0 4.78565034 5.54604923 6.25675682 6.89493039 7.44005852 7.87462034 8.18464867 8.36017895 8.39556949 8.28968281 8.0459222 7.67212232 7.1802974 6.58625511 5.90908845 5.17056212 4.39441296 3.60558704 2.82943788 2.09091155 1.41374489 0.8197026 0.32787768 -0.0459222 -0.28968281 -0.39556949 -0.36017895 -0.18464867 0.12537966 0.55994148 1.10506961 1.74324318 2.45395077 3.21434966],
            [0.20000000000000018 0.33704175437357 0.7347524157501475 1.354201293581075 2.134752415750147 2.9999999999999996 -0.5999999999999996 -0.5097404838545647 -0.2434879244487087 0.18540666311509257 0.7554367133085589 1.4380185391767903 2.1989246377572673 2.9999999999999996 -1.4000000000000004 -1.3331541132537161 -1.1346475314579978 -0.8105117766515302 -0.37059554972350384 0.17173451737922596 0.8000000000000016 1.4951113693670546 2.2359480182655065 2.999999999999999;
             -3.9999999999999996 -4.865247584249852 -5.645798706418924 -6.2652475842498525 -6.662958245626429 -6.8 -3.9999999999999996 -4.801075362242731 -5.561981460823208 -6.24456328669144 -6.814593336884906 -7.243487924448708 -7.509740483854564 -7.6 -3.9999999999999996 -4.764051981734492 -5.504888630632941 -6.200000000000001 -6.828265482620774 -7.370595549723503 -7.810511776651531 -8.134647531457997 -8.333154113253716 -8.4],
        ]

        @testset "$(shape_names[i])" for i in eachindex(shape_names)
            @test isapprox(shapes[i].coordinates, expected_coords[i]; atol=1e-4)
        end
    end

    @testset verbose=true "SphereShape 3D" begin
        shape_names = [
            "1-particle VoxelSphere",
            "7-particle VoxelSphere",
            "1-particle RoundSphere",
            "2-particle RoundSphere",
            "3-particle RoundSphere",
            "4-particle RoundSphere",
            "5-particle RoundSphere",
            "6-particle RoundSphere",
            "VoxelSphere with Cutout and tlsph",
            "VoxelSphere with Two Layers and tlsph",
            "VoxelSphere with Two Layers Outwards and tlsph",
            "RoundSphere with Two Layers",
            "RoundSphere with Two Layers Outwards and tlsph",
        ]

        shapes = [
            SphereShape(1.0, 1.0, (0.2, -1.0, 0.3), 1000.0),
            SphereShape(1.0, 1.5, (0.0, 0.0, 0.0), 1000.0),
            SphereShape(1.0, 0.8, (0.2, -1.0, 0.3), 1000.0, sphere_type=RoundSphere()),
            SphereShape(1.0, 0.9, (0.2, -1.0, 0.3), 1000.0, sphere_type=RoundSphere()),
            SphereShape(1.0, 1.0, (0.2, -1.0, 0.3), 1000.0, sphere_type=RoundSphere()),
            SphereShape(1.0, 1.05, (0.2, -1.0, 0.3), 1000.0, sphere_type=RoundSphere()),
            SphereShape(1.0, 1.1, (0.2, -1.0, 0.3), 1000.0, sphere_type=RoundSphere()),
            SphereShape(1.0, 1.2, (-0.3, 0.1, 0.8), 1000.0, sphere_type=RoundSphere()),
            SphereShape(0.1, 0.5, (0.3, 0.4, 0.5), 1000.0, cutout_min=(0.18, 0.4, 0.5),
                        cutout_max=(0.42, 10.0, 1.0), tlsph=true),
            SphereShape(0.1, 0.5, (0.3, 0.4, 0.5), 1000.0, n_layers=2, tlsph=true),
            SphereShape(0.1, 0.5, (0.3, 0.4, 0.5), 1000.0, n_layers=2,
                        layer_outwards=true, tlsph=true),
            SphereShape(0.1, 0.5, (0.3, 0.4, 0.5), 1000.0, n_layers=2,
                        sphere_type=RoundSphere()),
            SphereShape(0.1, 0.55, (0.3, 0.4, 0.5), 1000.0, n_layers=2, layer_outwards=true,
                        sphere_type=RoundSphere(), tlsph=true),
        ]

        expected_coords = [
            [0.2, -1.0, 0.3],
            [0.0 0.0 -1.0 0.0 1.0 0.0 0.0; 0.0 -1.0 0.0 0.0 0.0 1.0 0.0;
             -1.0 0.0 0.0 0.0 0.0 0.0 1.0],
            [0.2, -1.0, 0.3],
            [-0.2 0.6; -1.0 -1.0; 0.3 0.3],
            [0.7 -0.05 -0.05; -1.0 -0.5669873 -1.4330127; 0.3 0.3 0.3],
            [0.51754265 -0.11754265 -0.11754265 0.51754265;
             -0.68245735 -1.31754265 -0.68245735 -1.31754265;
             0.61754265 0.61754265 -0.01754265 -0.01754265],
            [0.2 0.8 -0.1 -0.1 0.2; -1.0 -1.0 -0.48038476 -1.51961524 -1.0;
             0.9 0.3 0.3 0.3 -0.3],
            [-0.3 0.4 -0.3 -1.0 -0.3 -0.3; 0.1 0.1 0.8 0.1 -0.6 0.1;
             1.5 0.8 0.8 0.8 0.8 0.1],
            [0.3 0.3 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 0.3 0.3 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.3 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.3 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.1 0.5 0.6 0.7 0.8 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.0 0.1 0.5 0.6 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.0 0.1 0.5 0.6 0.1 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.0 0.1 0.5 0.6 0.1 0.5 0.3 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.0 0.1 0.5 0.6 -0.0 0.1 0.5 0.6 0.1 0.5 0.3 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.5 0.6 0.1 0.5 0.1 0.5;
             0.4 0.1 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.7 0.0 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.8 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.8 0.8 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.8 0.8 0.0 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.1 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.6 0.6;
             0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9],
            [0.3 0.3 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 0.3 0.3 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.3 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.3 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.2 -0.1 0.7 0.8 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.3 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.6 0.7 -0.1 -0.0 0.1 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.3 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.3 0.3 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 0.3 0.3;
             0.4 0.1 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.7 0.0 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.8 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.0 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.8 0.1 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.7 0.4;
             0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0],
            [0.3 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.2 -0.1 0.7 0.8 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.3 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.2 -0.1 0.7 0.8 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.4 -0.3 0.9 1.0 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.2 -0.1 0.7 0.8 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.3 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.3 -0.2 -0.1 0.7 0.8 0.9 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.2 -0.1 0.7 0.8 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.2 -0.1 -0.0 0.6 0.7 0.8 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.1 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.1 0.2 0.3 0.4 0.5 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 -0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.1 0.2 0.3 0.4 0.5 0.3;
             0.4 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 -0.2 -0.2 -0.2 -0.2 -0.2 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0 1.0 1.0 1.0 1.0 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0 1.0 1.0 1.0 1.0 1.0 1.0 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0 1.0 1.0 1.0 1.0 1.0 1.0 -0.3 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.1 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0 1.0 1.0 1.0 1.0 1.0 1.0 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.2 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0 1.0 1.0 1.0 1.0 1.0 1.0 -0.2 -0.2 -0.2 -0.2 -0.2 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0 1.0 1.0 1.0 1.0 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.4;
             -0.2 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 -0.1 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.0 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.1 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.2 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.3 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.4 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.6 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.7 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.8 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 0.9 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.0 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.1 1.2],
            [0.3 0.41091791 0.36915619 0.27531844 0.20006642 0.20006642 0.27531844 0.36915619 0.49858033 0.47197561 0.39929017 0.3 0.20070983 0.12802439 0.10141967 0.12802439 0.20070983 0.3 0.39929017 0.47197561 0.56884782 0.54838299 0.49010411 0.40288361 0.3 0.19711639 0.10989589 0.05161701 0.03115218 0.05161701 0.10989589 0.19711639 0.3 0.40288361 0.49010411 0.54838299 0.61993839 0.60427949 0.55883559 0.48805507 0.3988664 0.3 0.2011336 0.11194493 0.04116441 -0.00427949 -0.01993839 -0.00427949 0.04116441 0.11194493 0.2011336 0.3 0.3988664 0.48805507 0.55883559 0.60427949 0.64668776 0.6312854 0.58644687 0.51615628 0.42665926 0.32590801 0.22285472 0.12665612 0.04585989 -0.01235488 -0.04281555 -0.04281555 -0.01235488 0.04585989 0.12665612 0.22285472 0.32590801 0.42665926 0.51615628 0.58644687 0.6312854 0.64668776 0.6312854 0.58644687 0.51615628 0.42665926 0.32590801 0.22285472 0.12665612 0.04585989 -0.01235488 -0.04281555 -0.04281555 -0.01235488 0.04585989 0.12665612 0.22285472 0.32590801 0.42665926 0.51615628 0.58644687 0.6312854 0.61993839 0.60427949 0.55883559 0.48805507 0.3988664 0.3 0.2011336 0.11194493 0.04116441 -0.00427949 -0.01993839 -0.00427949 0.04116441 0.11194493 0.2011336 0.3 0.3988664 0.48805507 0.55883559 0.60427949 0.56884782 0.54838299 0.49010411 0.40288361 0.3 0.19711639 0.10989589 0.05161701 0.03115218 0.05161701 0.10989589 0.19711639 0.3 0.40288361 0.49010411 0.54838299 0.49858033 0.47197561 0.39929017 0.3 0.20070983 0.12802439 0.10141967 0.12802439 0.20070983 0.3 0.39929017 0.47197561 0.41091791 0.36915619 0.27531844 0.20006642 0.20006642 0.27531844 0.36915619 0.3 0.3 0.41154685 0.36954832 0.27517849 0.19949976 0.19949976 0.27517849 0.36954832 0.505355 0.48183282 0.41665493 0.32475281 0.22718012 0.14628958 0.10061225 0.10061225 0.14628958 0.22718012 0.32475281 0.41665493 0.48183282 0.58681549 0.56744748 0.51195921 0.42784467 0.326464 0.22150921 0.12715505 0.05614455 0.01806809 0.01806809 0.05614455 0.12715505 0.22150921 0.326464 0.42784467 0.51195921 0.56744748 0.65420965 0.63986167 0.59798012 0.53195799 0.447144 0.35040929 0.24959071 0.152856 0.06804201 0.00201988 -0.03986167 -0.05420965 -0.03986167 0.00201988 0.06804201 0.152856 0.24959071 0.35040929 0.447144 0.53195799 0.59798012 0.63986167 0.7068799 0.69505671 0.66027426 0.60455398 0.53113413 0.4442816 0.34904395 0.25095605 0.1557184 0.06886587 -0.00455398 -0.06027426 -0.09505671 -0.1068799 -0.09505671 -0.06027426 -0.00455398 0.06886587 0.1557184 0.25095605 0.34904395 0.4442816 0.53113413 0.60455398 0.66027426 0.69505671 0.7392343 0.72739468 0.6925141 0.63647299 0.56229253 0.47397182 0.37627224 0.2744608 0.17402618 0.08038285 -0.00142086 -0.0669749 -0.11274523 -0.13626435 -0.13626435 -0.11274523 -0.0669749 -0.00142086 0.08038285 0.17402618 0.2744608 0.37627224 0.47397182 0.56229253 0.63647299 0.6925141 0.72739468 0.75 0.73871756 0.70543599 0.65182417 0.58057041 0.49524768 0.40013442 0.3 0.19986558 0.10475232 0.01942959 -0.05182417 -0.10543599 -0.13871756 -0.15 -0.13871756 -0.10543599 -0.05182417 0.01942959 0.10475232 0.19986558 0.3 0.40013442 0.49524768 0.58057041 0.65182417 0.70543599 0.73871756 0.7392343 0.72739468 0.6925141 0.63647299 0.56229253 0.47397182 0.37627224 0.2744608 0.17402618 0.08038285 -0.00142086 -0.0669749 -0.11274523 -0.13626435 -0.13626435 -0.11274523 -0.0669749 -0.00142086 0.08038285 0.17402618 0.2744608 0.37627224 0.47397182 0.56229253 0.63647299 0.6925141 0.72739468 0.7068799 0.69505671 0.66027426 0.60455398 0.53113413 0.4442816 0.34904395 0.25095605 0.1557184 0.06886587 -0.00455398 -0.06027426 -0.09505671 -0.1068799 -0.09505671 -0.06027426 -0.00455398 0.06886587 0.1557184 0.25095605 0.34904395 0.4442816 0.53113413 0.60455398 0.66027426 0.69505671 0.65420965 0.63986167 0.59798012 0.53195799 0.447144 0.35040929 0.24959071 0.152856 0.06804201 0.00201988 -0.03986167 -0.05420965 -0.03986167 0.00201988 0.06804201 0.152856 0.24959071 0.35040929 0.447144 0.53195799 0.59798012 0.63986167 0.58681549 0.56744748 0.51195921 0.42784467 0.326464 0.22150921 0.12715505 0.05614455 0.01806809 0.01806809 0.05614455 0.12715505 0.22150921 0.326464 0.42784467 0.51195921 0.56744748 0.505355 0.48183282 0.41665493 0.32475281 0.22718012 0.14628958 0.10061225 0.10061225 0.14628958 0.22718012 0.32475281 0.41665493 0.48183282 0.41154685 0.36954832 0.27517849 0.19949976 0.19949976 0.27517849 0.36954832 0.3;
             0.4 0.4 0.48671911 0.50813697 0.44812548 0.35187452 0.29186303 0.31328089 0.4 0.49929017 0.57197561 0.59858033 0.57197561 0.49929017 0.4 0.30070983 0.22802439 0.20141967 0.22802439 0.30070983 0.4 0.50288361 0.59010411 0.64838299 0.66884782 0.64838299 0.59010411 0.50288361 0.4 0.29711639 0.20989589 0.15161701 0.13115218 0.15161701 0.20989589 0.29711639 0.4 0.4988664 0.58805507 0.65883559 0.70427949 0.71993839 0.70427949 0.65883559 0.58805507 0.4988664 0.4 0.3011336 0.21194493 0.14116441 0.09572051 0.08006161 0.09572051 0.14116441 0.21194493 0.3011336 0.4 0.50218801 0.59529617 0.67105141 0.72272254 0.74571835 0.73799558 0.70024041 0.63580756 0.55042218 0.45167113 0.34832887 0.24957782 0.16419244 0.09975959 0.06200442 0.05428165 0.07727746 0.12894859 0.20470383 0.29781199 0.4 0.50218801 0.59529617 0.67105141 0.72272254 0.74571835 0.73799558 0.70024041 0.63580756 0.55042218 0.45167113 0.34832887 0.24957782 0.16419244 0.09975959 0.06200442 0.05428165 0.07727746 0.12894859 0.20470383 0.29781199 0.4 0.4988664 0.58805507 0.65883559 0.70427949 0.71993839 0.70427949 0.65883559 0.58805507 0.4988664 0.4 0.3011336 0.21194493 0.14116441 0.09572051 0.08006161 0.09572051 0.14116441 0.21194493 0.3011336 0.4 0.50288361 0.59010411 0.64838299 0.66884782 0.64838299 0.59010411 0.50288361 0.4 0.29711639 0.20989589 0.15161701 0.13115218 0.15161701 0.20989589 0.29711639 0.4 0.49929017 0.57197561 0.59858033 0.57197561 0.49929017 0.4 0.30070983 0.22802439 0.20141967 0.22802439 0.30070983 0.4 0.48671911 0.50813697 0.44812548 0.35187452 0.29186303 0.31328089 0.4 0.4 0.4 0.48721084 0.50875014 0.44839836 0.35160164 0.29124986 0.31278916 0.4 0.49543322 0.56900385 0.60385773 0.59201026 0.53617555 0.44914467 0.35085533 0.26382445 0.20798974 0.19614227 0.23099615 0.30456678 0.4 0.50360971 0.59322635 0.6567467 0.68559199 0.6758665 0.6288837 0.5509889 0.45270221 0.34729779 0.2490111 0.1711163 0.1241335 0.11440801 0.1432533 0.20677365 0.29639029 0.4 0.49979239 0.59150019 0.66769379 0.72220043 0.7506043 0.7506043 0.72220043 0.66769379 0.59150019 0.49979239 0.4 0.30020761 0.20849981 0.13230621 0.07779957 0.0493957 0.0493957 0.07779957 0.13230621 0.20849981 0.30020761 0.4 0.49737273 0.58908652 0.66981128 0.73485559 0.78043932 0.80391329 0.80391329 0.78043932 0.73485559 0.66981128 0.58908652 0.49737273 0.4 0.30262727 0.21091348 0.13018872 0.06514441 0.01956068 -0.00391329 -0.00391329 0.01956068 0.06514441 0.13018872 0.21091348 0.30262727 0.4 0.5012944 0.59712799 0.68233436 0.75232002 0.80331201 0.83256134 0.83849118 0.82078185 0.78038806 0.71948745 0.64136319 0.55022698 0.45099199 0.34900801 0.24977302 0.15863681 0.08051255 0.01961194 -0.02078185 -0.03849118 -0.03256134 -0.00331201 0.04767998 0.11766564 0.20287201 0.2987056 0.4 0.50013442 0.59524768 0.68057041 0.75182417 0.80543599 0.83871756 0.85 0.83871756 0.80543599 0.75182417 0.68057041 0.59524768 0.50013442 0.4 0.29986558 0.20475232 0.11942959 0.04817583 -0.00543599 -0.03871756 -0.05 -0.03871756 -0.00543599 0.04817583 0.11942959 0.20475232 0.29986558 0.4 0.5012944 0.59712799 0.68233436 0.75232002 0.80331201 0.83256134 0.83849118 0.82078185 0.78038806 0.71948745 0.64136319 0.55022698 0.45099199 0.34900801 0.24977302 0.15863681 0.08051255 0.01961194 -0.02078185 -0.03849118 -0.03256134 -0.00331201 0.04767998 0.11766564 0.20287201 0.2987056 0.4 0.49737273 0.58908652 0.66981128 0.73485559 0.78043932 0.80391329 0.80391329 0.78043932 0.73485559 0.66981128 0.58908652 0.49737273 0.4 0.30262727 0.21091348 0.13018872 0.06514441 0.01956068 -0.00391329 -0.00391329 0.01956068 0.06514441 0.13018872 0.21091348 0.30262727 0.4 0.49979239 0.59150019 0.66769379 0.72220043 0.7506043 0.7506043 0.72220043 0.66769379 0.59150019 0.49979239 0.4 0.30020761 0.20849981 0.13230621 0.07779957 0.0493957 0.0493957 0.07779957 0.13230621 0.20849981 0.30020761 0.4 0.50360971 0.59322635 0.6567467 0.68559199 0.6758665 0.6288837 0.5509889 0.45270221 0.34729779 0.2490111 0.1711163 0.1241335 0.11440801 0.1432533 0.20677365 0.29639029 0.4 0.49543322 0.56900385 0.60385773 0.59201026 0.53617555 0.44914467 0.35085533 0.26382445 0.20798974 0.19614227 0.23099615 0.30456678 0.4 0.48721084 0.50875014 0.44839836 0.35160164 0.29124986 0.31278916 0.4;
             0.85 0.83195966 0.83195966 0.83195966 0.83195966 0.83195966 0.83195966 0.83195966 0.78821147 0.78821147 0.78821147 0.78821147 0.78821147 0.78821147 0.78821147 0.78821147 0.78821147 0.78821147 0.78821147 0.78821147 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.72410009 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.64191345 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.54803744 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.45196256 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.35808655 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.27589991 0.21178853 0.21178853 0.21178853 0.21178853 0.21178853 0.21178853 0.21178853 0.21178853 0.21178853 0.21178853 0.21178853 0.21178853 0.16804034 0.16804034 0.16804034 0.16804034 0.16804034 0.16804034 0.16804034 0.15 0.95 0.93595562 0.93595562 0.93595562 0.93595562 0.93595562 0.93595562 0.93595562 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.90041145 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.84675189 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.77755275 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.69222057 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5978429 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.4021571 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.30777943 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.22244725 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.15324811 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.09958855 0.06404438 0.06404438 0.06404438 0.06404438 0.06404438 0.06404438 0.06404438 0.05],
            [0.3 0.41157406 0.36956529 0.27517244 0.19947525 0.19947525 0.27517244 0.36956529 0.50686098 0.4831663 0.41751043 0.32493434 0.22664609 0.14516233 0.09915002 0.09915002 0.14516233 0.22664609 0.32493434 0.41751043 0.4831663 0.59629658 0.58024241 0.53381963 0.46205887 0.37273651 0.27553202 0.18097902 0.09932379 0.03941494 0.00774452 0.00774452 0.03941494 0.09932379 0.18097902 0.27553202 0.37273651 0.46205887 0.53381963 0.58024241 0.6763414 0.66351787 0.62592121 0.56611355 0.4881707 0.39740432 0.3 0.20259568 0.1118293 0.03388645 -0.02592121 -0.06351787 -0.0763414 -0.06351787 -0.02592121 0.03388645 0.1118293 0.20259568 0.3 0.39740432 0.4881707 0.56611355 0.62592121 0.66351787 0.74214449 0.73022643 0.69511475 0.63870233 0.56403038 0.47512449 0.37677759 0.27429158 0.17319153 0.07892775 -0.00341796 -0.06940633 -0.11547992 -0.13915487 -0.13915487 -0.11547992 -0.06940633 -0.00341796 0.07892775 0.17319153 0.27429158 0.37677759 0.47512449 0.56403038 0.63870233 0.69511475 0.73022643 0.79397679 0.78448516 0.75637504 0.71072669 0.64929434 0.5744388 0.48903673 0.39637009 0.3 0.20362991 0.11096327 0.0255612 -0.04929434 -0.11072669 -0.15637504 -0.18448516 -0.19397679 -0.18448516 -0.15637504 -0.11072669 -0.04929434 0.0255612 0.11096327 0.20362991 0.3 0.39637009 0.48903673 0.5744388 0.64929434 0.71072669 0.75637504 0.78448516 0.82980744 0.82023313 0.79185624 0.74570238 0.68343968 0.60731846 0.52008996 0.42490684 0.32520925 0.22460054 0.12671696 0.03509628 -0.04695009 -0.11645678 -0.17091163 -0.20834651 -0.22740843 -0.22740843 -0.20834651 -0.17091163 -0.11645678 -0.04695009 0.03509628 0.12671696 0.22460054 0.32520925 0.42490684 0.52008996 0.60731846 0.68343968 0.74570238 0.79185624 0.82023313 0.8477922 0.83898895 0.81286216 0.77025155 0.71252668 0.64154285 0.55958156 0.4692771 0.37353194 0.2754234 0.17810477 0.08470395 -0.00177707 -0.07855873 -0.1431732 -0.19354372 -0.22805134 -0.24558695 -0.24558695 -0.22805134 -0.19354372 -0.1431732 -0.07855873 -0.00177707 0.08470395 0.17810477 0.2754234 0.37353194 0.4692771 0.55958156 0.64154285 0.71252668 0.77025155 0.81286216 0.83898895 0.84766126 0.83833629 0.81067892 0.76563099 0.70472655 0.63003964 0.54411363 0.44987462 0.35053181 0.24946819 0.15012538 0.05588637 -0.03003964 -0.10472655 -0.16563099 -0.21067892 -0.23833629 -0.24766126 -0.23833629 -0.21067892 -0.16563099 -0.10472655 -0.03003964 0.05588637 0.15012538 0.24946819 0.35053181 0.44987462 0.54411363 0.63003964 0.70472655 0.76563099 0.81067892 0.83833629 0.82980744 0.82023313 0.79185624 0.74570238 0.68343968 0.60731846 0.52008996 0.42490684 0.32520925 0.22460054 0.12671696 0.03509628 -0.04695009 -0.11645678 -0.17091163 -0.20834651 -0.22740843 -0.22740843 -0.20834651 -0.17091163 -0.11645678 -0.04695009 0.03509628 0.12671696 0.22460054 0.32520925 0.42490684 0.52008996 0.60731846 0.68343968 0.74570238 0.79185624 0.82023313 0.79397679 0.78448516 0.75637504 0.71072669 0.64929434 0.5744388 0.48903673 0.39637009 0.3 0.20362991 0.11096327 0.0255612 -0.04929434 -0.11072669 -0.15637504 -0.18448516 -0.19397679 -0.18448516 -0.15637504 -0.11072669 -0.04929434 0.0255612 0.11096327 0.20362991 0.3 0.39637009 0.48903673 0.5744388 0.64929434 0.71072669 0.75637504 0.78448516 0.74214449 0.73022643 0.69511475 0.63870233 0.56403038 0.47512449 0.37677759 0.27429158 0.17319153 0.07892775 -0.00341796 -0.06940633 -0.11547992 -0.13915487 -0.13915487 -0.11547992 -0.06940633 -0.00341796 0.07892775 0.17319153 0.27429158 0.37677759 0.47512449 0.56403038 0.63870233 0.69511475 0.73022643 0.6763414 0.66351787 0.62592121 0.56611355 0.4881707 0.39740432 0.3 0.20259568 0.1118293 0.03388645 -0.02592121 -0.06351787 -0.0763414 -0.06351787 -0.02592121 0.03388645 0.1118293 0.20259568 0.3 0.39740432 0.4881707 0.56611355 0.62592121 0.66351787 0.59629658 0.58024241 0.53381963 0.46205887 0.37273651 0.27553202 0.18097902 0.09932379 0.03941494 0.00774452 0.00774452 0.03941494 0.09932379 0.18097902 0.27553202 0.37273651 0.46205887 0.53381963 0.58024241 0.50686098 0.4831663 0.41751043 0.32493434 0.22664609 0.14516233 0.09915002 0.09915002 0.14516233 0.22664609 0.32493434 0.41751043 0.4831663 0.41157406 0.36956529 0.27517244 0.19947525 0.19947525 0.27517244 0.36956529 0.3 0.3 0.41173428 0.36966519 0.27513678 0.19933089 0.19933089 0.27513678 0.36966519 0.50797533 0.48415301 0.41814345 0.32506866 0.22625093 0.14432823 0.09806805 0.09806805 0.14432823 0.22625093 0.32506866 0.41814345 0.48415301 0.60190095 0.58712487 0.544243 0.47745293 0.39329252 0.3 0.20670748 0.12254707 0.055757 0.01287513 -0.00190095 0.01287513 0.055757 0.12254707 0.20670748 0.3 0.39329252 0.47745293 0.544243 0.58712487 0.68795667 0.67473737 0.63598033 0.57432679 0.49397834 0.40041058 0.3 0.19958942 0.10602166 0.02567321 -0.03598033 -0.07473737 -0.08795667 -0.07473737 -0.03598033 0.02567321 0.10602166 0.19958942 0.3 0.40041058 0.49397834 0.57432679 0.63598033 0.67473737 0.76366294 0.75353079 0.72357717 0.6751112 0.61025106 0.53183147 0.44327973 0.34846597 0.25153403 0.15672027 0.06816853 -0.01025106 -0.0751112 -0.12357717 -0.15353079 -0.16366294 -0.15353079 -0.12357717 -0.0751112 -0.01025106 0.06816853 0.15672027 0.25153403 0.34846597 0.44327973 0.53183147 0.61025106 0.6751112 0.72357717 0.75353079 0.82856536 0.8190135 0.79070313 0.74465748 0.68254074 0.60659799 0.51957399 0.42461401 0.32515015 0.22477731 0.1271232 0.03571732 -0.0461367 -0.11548044 -0.16980763 -0.20715475 -0.22617198 -0.22617198 -0.20715475 -0.16980763 -0.11548044 -0.0461367 0.03571732 0.1271232 0.22477731 0.32515015 0.42461401 0.51957399 0.60659799 0.68254074 0.74465748 0.79070313 0.8190135 0.8803953 0.87204684 0.84724163 0.80669328 0.75156829 0.6834525 0.60430548 0.51640415 0.42227726 0.32463267 0.22627945 0.13004704 0.03870386 -0.0451223 -0.11901992 -0.1808631 -0.22887273 -0.26166765 -0.27830442 -0.27830442 -0.26166765 -0.22887273 -0.1808631 -0.11901992 -0.0451223 0.03870386 0.13004704 0.22627945 0.32463267 0.42227726 0.51640415 0.60430548 0.6834525 0.75156829 0.80669328 0.84724163 0.87204684 0.91898777 0.91136701 0.88869236 0.85152215 0.80077163 0.73769045 0.66383189 0.58101457 0.49127774 0.39683102 0.3 0.20316898 0.10872226 0.01898543 -0.06383189 -0.13769045 -0.20077163 -0.25152215 -0.28869236 -0.31136701 -0.31898777 -0.31136701 -0.28869236 -0.25152215 -0.20077163 -0.13769045 -0.06383189 0.01898543 0.10872226 0.20316898 0.3 0.39683102 0.49127774 0.58101457 0.66383189 0.73769045 0.80077163 0.85152215 0.88869236 0.91136701 0.94220053 0.93429398 0.910769 0.87220486 0.81955114 0.75410435 0.677476 0.59155294 0.49845088 0.4004623 0.3 0.1995377 0.10154912 0.00844706 -0.077476 -0.15410435 -0.21955114 -0.27220486 -0.310769 -0.33429398 -0.34220053 -0.33429398 -0.310769 -0.27220486 -0.21955114 -0.15410435 -0.077476 0.00844706 0.10154912 0.1995377 0.3 0.4004623 0.49845088 0.59155294 0.677476 0.75410435 0.81955114 0.87220486 0.910769 0.93429398 0.95 0.94274004 0.92112232 0.88562976 0.8370552 0.77648372 0.70526837 0.625 0.53747167 0.44463861 0.34857456 0.25142544 0.15536139 0.06252833 -0.025 -0.10526837 -0.17648372 -0.2370552 -0.28562976 -0.32112232 -0.34274004 -0.35 -0.34274004 -0.32112232 -0.28562976 -0.2370552 -0.17648372 -0.10526837 -0.025 0.06252833 0.15536139 0.25142544 0.34857456 0.44463861 0.53747167 0.625 0.70526837 0.77648372 0.8370552 0.88562976 0.92112232 0.94274004 0.94220053 0.93429398 0.910769 0.87220486 0.81955114 0.75410435 0.677476 0.59155294 0.49845088 0.4004623 0.3 0.1995377 0.10154912 0.00844706 -0.077476 -0.15410435 -0.21955114 -0.27220486 -0.310769 -0.33429398 -0.34220053 -0.33429398 -0.310769 -0.27220486 -0.21955114 -0.15410435 -0.077476 0.00844706 0.10154912 0.1995377 0.3 0.4004623 0.49845088 0.59155294 0.677476 0.75410435 0.81955114 0.87220486 0.910769 0.93429398 0.91898777 0.91136701 0.88869236 0.85152215 0.80077163 0.73769045 0.66383189 0.58101457 0.49127774 0.39683102 0.3 0.20316898 0.10872226 0.01898543 -0.06383189 -0.13769045 -0.20077163 -0.25152215 -0.28869236 -0.31136701 -0.31898777 -0.31136701 -0.28869236 -0.25152215 -0.20077163 -0.13769045 -0.06383189 0.01898543 0.10872226 0.20316898 0.3 0.39683102 0.49127774 0.58101457 0.66383189 0.73769045 0.80077163 0.85152215 0.88869236 0.91136701 0.8803953 0.87204684 0.84724163 0.80669328 0.75156829 0.6834525 0.60430548 0.51640415 0.42227726 0.32463267 0.22627945 0.13004704 0.03870386 -0.0451223 -0.11901992 -0.1808631 -0.22887273 -0.26166765 -0.27830442 -0.27830442 -0.26166765 -0.22887273 -0.1808631 -0.11901992 -0.0451223 0.03870386 0.13004704 0.22627945 0.32463267 0.42227726 0.51640415 0.60430548 0.6834525 0.75156829 0.80669328 0.84724163 0.87204684 0.82856536 0.8190135 0.79070313 0.74465748 0.68254074 0.60659799 0.51957399 0.42461401 0.32515015 0.22477731 0.1271232 0.03571732 -0.0461367 -0.11548044 -0.16980763 -0.20715475 -0.22617198 -0.22617198 -0.20715475 -0.16980763 -0.11548044 -0.0461367 0.03571732 0.1271232 0.22477731 0.32515015 0.42461401 0.51957399 0.60659799 0.68254074 0.74465748 0.79070313 0.8190135 0.76366294 0.75353079 0.72357717 0.6751112 0.61025106 0.53183147 0.44327973 0.34846597 0.25153403 0.15672027 0.06816853 -0.01025106 -0.0751112 -0.12357717 -0.15353079 -0.16366294 -0.15353079 -0.12357717 -0.0751112 -0.01025106 0.06816853 0.15672027 0.25153403 0.34846597 0.44327973 0.53183147 0.61025106 0.6751112 0.72357717 0.75353079 0.68795667 0.67473737 0.63598033 0.57432679 0.49397834 0.40041058 0.3 0.19958942 0.10602166 0.02567321 -0.03598033 -0.07473737 -0.08795667 -0.07473737 -0.03598033 0.02567321 0.10602166 0.19958942 0.3 0.40041058 0.49397834 0.57432679 0.63598033 0.67473737 0.60190095 0.58712487 0.544243 0.47745293 0.39329252 0.3 0.20670748 0.12254707 0.055757 0.01287513 -0.00190095 0.01287513 0.055757 0.12254707 0.20670748 0.3 0.39329252 0.47745293 0.544243 0.58712487 0.50797533 0.48415301 0.41814345 0.32506866 0.22625093 0.14432823 0.09806805 0.09806805 0.14432823 0.22625093 0.32506866 0.41814345 0.48415301 0.41173428 0.36966519 0.27513678 0.19933089 0.19933089 0.27513678 0.36966519 0.3;
             0.4 0.4 0.48723211 0.50877666 0.44841017 0.35158983 0.29122334 0.31276789 0.4 0.49613309 0.57024325 0.60535273 0.59341838 0.5371742 0.44950507 0.35049493 0.2628258 0.20658162 0.19464727 0.22975675 0.30386691 0.4 0.49620734 0.58198912 0.64804956 0.68722998 0.69528457 0.6713405 0.61799248 0.54102158 0.44876881 0.35123119 0.25897842 0.18200752 0.1286595 0.10471543 0.11277002 0.15195044 0.21801088 0.30379266 0.4 0.49740432 0.5881707 0.66611355 0.72592121 0.76351787 0.7763414 0.76351787 0.72592121 0.66611355 0.5881707 0.49740432 0.4 0.30259568 0.2118293 0.13388645 0.07407879 0.03648213 0.0236586 0.03648213 0.07407879 0.13388645 0.2118293 0.30259568 0.4 0.50196554 0.59843409 0.684205 0.75465435 0.80598419 0.83542732 0.84139645 0.82356979 0.78290836 0.72160425 0.64296237 0.55122232 0.45132984 0.34867016 0.24877768 0.15703763 0.07839575 0.01709164 -0.02356979 -0.04139645 -0.03542732 -0.00598419 0.04534565 0.115795 0.20156591 0.29803446 0.4 0.49637009 0.58903673 0.6744388 0.74929434 0.81072669 0.85637504 0.88448516 0.89397679 0.88448516 0.85637504 0.81072669 0.74929434 0.6744388 0.58903673 0.49637009 0.4 0.30362991 0.21096327 0.1255612 0.05070566 -0.01072669 -0.05637504 -0.08448516 -0.09397679 -0.08448516 -0.05637504 -0.01072669 0.05070566 0.1255612 0.21096327 0.30362991 0.4 0.50026672 0.59690953 0.68643553 0.76560899 0.8315684 0.8819298 0.914873 0.92920735 0.92441476 0.90066846 0.8588267 0.80040175 0.72750523 0.64277182 0.549264 0.4503614 0.3496386 0.250736 0.15722818 0.07249477 -0.00040175 -0.0588267 -0.10066846 -0.12441476 -0.12920735 -0.114873 -0.0819298 -0.0315684 0.03439101 0.11356447 0.20309047 0.29973328 0.4 0.49781207 0.59248039 0.68096222 0.7604137 0.82828119 0.88238336 0.92098134 0.94283455 0.94724061 0.93405791 0.90371014 0.85717272 0.79594139 0.72198418 0.63767813 0.54573291 0.44910371 0.35089629 0.25426709 0.16232187 0.07801582 0.00405861 -0.05717272 -0.10371014 -0.13405791 -0.14724061 -0.14283455 -0.12098134 -0.08238336 -0.02828119 0.0395863 0.11903778 0.20751961 0.30218793 0.4 0.50063249 0.59783807 0.6883065 0.768957 0.83704312 0.89024626 0.92675464 0.94532503 0.94532503 0.92675464 0.89024626 0.83704312 0.768957 0.6883065 0.59783807 0.50063249 0.4 0.29936751 0.20216193 0.1116935 0.031043 -0.03704312 -0.09024626 -0.12675464 -0.14532503 -0.14532503 -0.12675464 -0.09024626 -0.03704312 0.031043 0.1116935 0.20216193 0.29936751 0.4 0.50026672 0.59690953 0.68643553 0.76560899 0.8315684 0.8819298 0.914873 0.92920735 0.92441476 0.90066846 0.8588267 0.80040175 0.72750523 0.64277182 0.549264 0.4503614 0.3496386 0.250736 0.15722818 0.07249477 -0.00040175 -0.0588267 -0.10066846 -0.12441476 -0.12920735 -0.114873 -0.0819298 -0.0315684 0.03439101 0.11356447 0.20309047 0.29973328 0.4 0.49637009 0.58903673 0.6744388 0.74929434 0.81072669 0.85637504 0.88448516 0.89397679 0.88448516 0.85637504 0.81072669 0.74929434 0.6744388 0.58903673 0.49637009 0.4 0.30362991 0.21096327 0.1255612 0.05070566 -0.01072669 -0.05637504 -0.08448516 -0.09397679 -0.08448516 -0.05637504 -0.01072669 0.05070566 0.1255612 0.21096327 0.30362991 0.4 0.50196554 0.59843409 0.684205 0.75465435 0.80598419 0.83542732 0.84139645 0.82356979 0.78290836 0.72160425 0.64296237 0.55122232 0.45132984 0.34867016 0.24877768 0.15703763 0.07839575 0.01709164 -0.02356979 -0.04139645 -0.03542732 -0.00598419 0.04534565 0.115795 0.20156591 0.29803446 0.4 0.49740432 0.5881707 0.66611355 0.72592121 0.76351787 0.7763414 0.76351787 0.72592121 0.66611355 0.5881707 0.49740432 0.4 0.30259568 0.2118293 0.13388645 0.07407879 0.03648213 0.0236586 0.03648213 0.07407879 0.13388645 0.2118293 0.30259568 0.4 0.49620734 0.58198912 0.64804956 0.68722998 0.69528457 0.6713405 0.61799248 0.54102158 0.44876881 0.35123119 0.25897842 0.18200752 0.1286595 0.10471543 0.11277002 0.15195044 0.21801088 0.30379266 0.4 0.49613309 0.57024325 0.60535273 0.59341838 0.5371742 0.44950507 0.35049493 0.2628258 0.20658162 0.19464727 0.22975675 0.30386691 0.4 0.48723211 0.50877666 0.44841017 0.35158983 0.29122334 0.31276789 0.4 0.4 0.4 0.48735738 0.50893287 0.44847969 0.35152031 0.29106713 0.31264262 0.4 0.49665096 0.57116034 0.60645896 0.59446031 0.53791315 0.44977175 0.35022825 0.26208685 0.20553969 0.19354104 0.22883966 0.30334904 0.4 0.49329252 0.57745293 0.644243 0.68712487 0.70190095 0.68712487 0.644243 0.57745293 0.49329252 0.4 0.30670748 0.22254707 0.155757 0.11287513 0.09809905 0.11287513 0.155757 0.22254707 0.30670748 0.4 0.50041058 0.59397834 0.67432679 0.73598033 0.77473737 0.78795667 0.77473737 0.73598033 0.67432679 0.59397834 0.50041058 0.4 0.29958942 0.20602166 0.12567321 0.06401967 0.02526263 0.01204333 0.02526263 0.06401967 0.12567321 0.20602166 0.29958942 0.4 0.49640095 0.58858871 0.67253424 0.74456871 0.80154388 0.84096966 0.86112295 0.86112295 0.84096966 0.80154388 0.74456871 0.67253424 0.58858871 0.49640095 0.4 0.30359905 0.21141129 0.12746576 0.05543129 -0.00154388 -0.04096966 -0.06112295 -0.06112295 -0.04096966 -0.00154388 0.05543129 0.12746576 0.21141129 0.30359905 0.4 0.50003165 0.5964479 0.68576401 0.76475186 0.83055663 0.88079996 0.91366593 0.92796668 0.92318533 0.8994947 0.85775103 0.79946305 0.72673743 0.64220267 0.54891407 0.45024333 0.34975667 0.25108593 0.15779733 0.07326257 0.00053695 -0.05775103 -0.0994947 -0.12318533 -0.12796668 -0.11366593 -0.08079996 -0.03055663 0.03524814 0.11423599 0.2035521 0.29996835 0.4 0.49808728 0.59335277 0.68305585 0.76461593 0.83568667 0.89422351 0.93854243 0.96736846 0.97987234 0.97569435 0.95495467 0.91824997 0.86663615 0.80159806 0.72500673 0.63906555 0.54624688 0.44922096 0.35077904 0.25375312 0.16093445 0.07499327 -0.00159806 -0.06663615 -0.11824997 -0.15495467 -0.17569435 -0.17987234 -0.16736846 -0.13854243 -0.09422351 -0.03568667 0.03538407 0.11694415 0.20664723 0.30191272 0.4 0.49683102 0.59127774 0.68101457 0.76383189 0.83769045 0.90077163 0.95152215 0.98869236 1.01136701 1.01898777 1.01136701 0.98869236 0.95152215 0.90077163 0.83769045 0.76383189 0.68101457 0.59127774 0.49683102 0.4 0.30316898 0.20872226 0.11898543 0.03616811 -0.03769045 -0.10077163 -0.15152215 -0.18869236 -0.21136701 -0.21898777 -0.21136701 -0.18869236 -0.15152215 -0.10077163 -0.03769045 0.03616811 0.11898543 0.20872226 0.30316898 0.4 0.5004623 0.59845088 0.69155294 0.777476 0.85410435 0.91955114 0.97220486 1.010769 1.03429398 1.04220053 1.03429398 1.010769 0.97220486 0.91955114 0.85410435 0.777476 0.69155294 0.59845088 0.5004623 0.4 0.2995377 0.20154912 0.10844706 0.022524 -0.05410435 -0.11955114 -0.17220486 -0.210769 -0.23429398 -0.24220053 -0.23429398 -0.210769 -0.17220486 -0.11955114 -0.05410435 0.022524 0.10844706 0.20154912 0.2995377 0.4 0.49687747 0.59159086 0.68202443 0.76615804 0.84211228 0.90819046 0.96291651 1.00506794 1.03370314 1.04818247 1.04818247 1.03370314 1.00506794 0.96291651 0.90819046 0.84211228 0.76615804 0.68202443 0.59159086 0.49687747 0.4 0.30312253 0.20840914 0.11797557 0.03384196 -0.04211228 -0.10819046 -0.16291651 -0.20506794 -0.23370314 -0.24818247 -0.24818247 -0.23370314 -0.20506794 -0.16291651 -0.10819046 -0.04211228 0.03384196 0.11797557 0.20840914 0.30312253 0.4 0.5004623 0.59845088 0.69155294 0.777476 0.85410435 0.91955114 0.97220486 1.010769 1.03429398 1.04220053 1.03429398 1.010769 0.97220486 0.91955114 0.85410435 0.777476 0.69155294 0.59845088 0.5004623 0.4 0.2995377 0.20154912 0.10844706 0.022524 -0.05410435 -0.11955114 -0.17220486 -0.210769 -0.23429398 -0.24220053 -0.23429398 -0.210769 -0.17220486 -0.11955114 -0.05410435 0.022524 0.10844706 0.20154912 0.2995377 0.4 0.49683102 0.59127774 0.68101457 0.76383189 0.83769045 0.90077163 0.95152215 0.98869236 1.01136701 1.01898777 1.01136701 0.98869236 0.95152215 0.90077163 0.83769045 0.76383189 0.68101457 0.59127774 0.49683102 0.4 0.30316898 0.20872226 0.11898543 0.03616811 -0.03769045 -0.10077163 -0.15152215 -0.18869236 -0.21136701 -0.21898777 -0.21136701 -0.18869236 -0.15152215 -0.10077163 -0.03769045 0.03616811 0.11898543 0.20872226 0.30316898 0.4 0.49808728 0.59335277 0.68305585 0.76461593 0.83568667 0.89422351 0.93854243 0.96736846 0.97987234 0.97569435 0.95495467 0.91824997 0.86663615 0.80159806 0.72500673 0.63906555 0.54624688 0.44922096 0.35077904 0.25375312 0.16093445 0.07499327 -0.00159806 -0.06663615 -0.11824997 -0.15495467 -0.17569435 -0.17987234 -0.16736846 -0.13854243 -0.09422351 -0.03568667 0.03538407 0.11694415 0.20664723 0.30191272 0.4 0.50003165 0.5964479 0.68576401 0.76475186 0.83055663 0.88079996 0.91366593 0.92796668 0.92318533 0.8994947 0.85775103 0.79946305 0.72673743 0.64220267 0.54891407 0.45024333 0.34975667 0.25108593 0.15779733 0.07326257 0.00053695 -0.05775103 -0.0994947 -0.12318533 -0.12796668 -0.11366593 -0.08079996 -0.03055663 0.03524814 0.11423599 0.2035521 0.29996835 0.4 0.49640095 0.58858871 0.67253424 0.74456871 0.80154388 0.84096966 0.86112295 0.86112295 0.84096966 0.80154388 0.74456871 0.67253424 0.58858871 0.49640095 0.4 0.30359905 0.21141129 0.12746576 0.05543129 -0.00154388 -0.04096966 -0.06112295 -0.06112295 -0.04096966 -0.00154388 0.05543129 0.12746576 0.21141129 0.30359905 0.4 0.50041058 0.59397834 0.67432679 0.73598033 0.77473737 0.78795667 0.77473737 0.73598033 0.67432679 0.59397834 0.50041058 0.4 0.29958942 0.20602166 0.12567321 0.06401967 0.02526263 0.01204333 0.02526263 0.06401967 0.12567321 0.20602166 0.29958942 0.4 0.49329252 0.57745293 0.644243 0.68712487 0.70190095 0.68712487 0.644243 0.57745293 0.49329252 0.4 0.30670748 0.22254707 0.155757 0.11287513 0.09809905 0.11287513 0.155757 0.22254707 0.30670748 0.4 0.49665096 0.57116034 0.60645896 0.59446031 0.53791315 0.44977175 0.35022825 0.26208685 0.20553969 0.19354104 0.22883966 0.30334904 0.4 0.48735738 0.50893287 0.44847969 0.35152031 0.29106713 0.31264262 0.4;
             1.05 1.03856404 1.03856404 1.03856404 1.03856404 1.03856404 1.03856404 1.03856404 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 1.00961607 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.96336631 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.90108248 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.82712115 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.74184072 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.64766204 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.54923114 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.449333 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.35233796 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.25815928 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.17287885 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.09891752 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 0.03663369 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.00961607 -0.03856404 -0.03856404 -0.03856404 -0.03856404 -0.03856404 -0.03856404 -0.03856404 -0.05 1.15 1.14032449 1.14032449 1.14032449 1.14032449 1.14032449 1.14032449 1.14032449 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.11582973 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.07563514 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 1.02152624 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.95553999 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.87831027 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.79264535 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.69837877 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.60039163 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.39960837 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.30162123 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.20735465 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.12168973 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 0.04446001 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.02152624 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.07563514 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.11582973 -0.14032449 -0.14032449 -0.14032449 -0.14032449 -0.14032449 -0.14032449 -0.14032449 -0.15],
        ]

        @testset "$(shape_names[i])" for i in eachindex(shape_names)
            @test isapprox(shapes[i].coordinates, expected_coords[i])
        end
    end
end
